import h from 'virtual-dom/h'
import Ajaxqueue from 'ajax_queue'
import AutoComplete from 'js-autocomplete'
import 'js-autocomplete/auto-complete.css'

function CityAutocomplete(args) {

	var o = this.options = extend({
		id: null,
		is_country: false,
		select: null,
	}, args)

	o.renderer.on.render.once(function() {
		new AutoComplete({
			selector: '#' + o.id + ' .autocomplete',
			cache: false, // disabled for now because it makes the suggestion data private to the autocomplete
			source: function(request, suggest){
				var el = document.querySelector('#' + o.id + ' .autocomplete')
				el.classList && el.classList.add('ui-autocomplete-loading') // no classList in MSIE9

				Ajaxqueue({
					url: '/backend/city-lookup/',
					name: 'autocomplete', // ensure single request
					json: {
						name: request,
						members: false,
						is_country: o.is_country
					},
					logErrors: 'City lookup',
					success: function (result) {
						self.suggestions = result.results

						suggest(o.is_country
							? Object.keys(suggestions)
							: self.suggestions.map(function(v){ return [v[0], v[1]].join(', ') })
						)
					},
					complete: clear_spinner
				})

				function clear_spinner() {
					el.classList && el.classList.remove('ui-autocomplete-loading')
				}

				clearTimeout(self.timeout)
				self.timeout = setTimeout(clear_spinner, 6000)
			},
			onSelect: function(e, term, item) {
				console.log('cityautocomplete.onSelect: item=', item)
				if (o.is_country) {
					o.value = item.getAttribute('data-val')
					o.select(self.suggestions[o.value])
				}else{
					var city = self.suggestions[element_index(item)]
					if(city){
						var pt = {
							lat: city[2],
							lon: city[3],
							title: city[0] + ', ' + city[1],
							long_title: city[0] + (city[6] ? (', ' + city[6]) : '')  + ', ' + city[1],
							altitude: city[5]
						}
						pt.label = pt.long_title;
						pt.value = pt.title;
						o.value = term
						o.select(pt)
					}
				}
			}
		})
	})
}

CityAutocomplete.prototype = {
	render: function() {
		return h('input.autocomplete', {type: 'text', placeholder: 'Enter a city name', value: this.options.value})
	}
}

function element_index(node) {
    var index = 0;
    while ((node = node.previousElementSibling)) {
        index++;
    }
    return index;
}

export default CityAutocomplete;
