define(['jquery', 'bus', '../widgets/widget', './widget_testable'], function($, Bus, DashboardWidget, WidgetTestable){

	var bus = Bus();

	function DashboardTestable(test){
		this.test = test;
		var dashboard_set = $('.dashboard').dashboardSet('instance');
		this.dashboard = dashboard_set.get_selected_dashboard(); // just get selected dashboard for now
		this.dashboard2 = dashboard_set.dashboard_lookup_by_name(this.dashboard.name);
		this.$dashboard = $('#' + this.dashboard.name);

		this.name = function(){
			return this.dashboard2.options.dashboard.name
		}

		this.title = function(){
			return this.dashboard2.options.dashboard.title
		}

		this.widgets = function(){
			return this.dashboard2.widgets
		}

		/*
		 *  Run an async callback for each widget in series
		 */
		this.iter = function(next, done){
			var fns = []
			this.dashboard2.widgets.all(function(w, c, r){
				fns.push(function(nxt){ next(w, c, r, nxt) })
			})
			fns.push(done)

			var i = 0;
			(function nxt(){
				var f = fns[i++]
				f && f(nxt)
			})()
		}

		this.first_widget = function(widget_type){
			if(!widget_type){
				var a = this.dashboard2.widgets.at(0).rows[0]
				if(a) return a.instance;
			}

			for(var c in this.dashboard2.widgets)

			// fallback in case col0 is empty - but may not be first widget, dom order is not guaranteed
			var el = $('body > .dashboard-widget:visible:first');
			var name = el.attr('id')
			var widget = this.find_widget_by_name(name)
			return widget;
		}

		this.n_widgets = function(){
			// returns the number of visible widgets (only widgets for the current dashboard should be visible)
			return $('body > .dashboard-widget:visible').length; // excludes any wrapped for slideup
		}

		this._check = function(){
			var self = this;
			var dashboard = dashboard_set.get_selected_dashboard2()
			if(dashboard.uuid !== this.dashboard2.uuid) test.fail('DST dashboard is not current')
			if(dashboard.options.dashboard.name !== this.dashboard2.options.dashboard.name) test.fail('DST dashboard is not current')
			var title = $('#dashboard-set .ui-tabs .ui-tabs-active').text()
			if(title !== dashboard.options.dashboard.title){
				throw new Error('selected tab is not selected! tab="' + title + '" (expected "' + dashboard.options.dashboard.title + '")')
			}

			var n1 = $('body > .dashboard-widget:visible').length; // excludes any wrapped for slideup
			var n2 = this.dashboard2.widgets.count();

			function find_widget(widget) {
				var d;
				$.each(dashboard_set.dashboards, function(i, dashboard){
					dashboard.widgets.all(function(w, c, r){
						if(w.instance.uuid === widget.uuid){
							d = dashboard
						}
					})
				})
				return d;
			}

			if(n1 !== n2){
				// TODO check the dashboard selection does not change midway through test
				console.log('inconsistency in widget count: model=' + n2 + ' visible=' + n1);
				if(!self._inconsistency){
					setTimeout(function(){
						if(self._inconsistency){
							throw new Error('inconsistency in widget count: model=' + n2 + ' visible=' + n1)
						}
					}, 4000)
					self._inconsistency = true
				}
				/*
				console.log('check: widgets for dashboard ' + this.dashboard2.options.dashboard.name + ':')
				// if phantom widgets with no 'li' and no 'instance' properties are observed, run the backend dashboard maintenance script
				this.dashboard2.widgets.all(function(v, c, r){
					console.log('    ' + c + '.' + r, v)
				})
				*/

				// some visible which dont exist - which ones are they?
				$('body > .dashboard-widget:visible').each(function(i, v){
					var widget = self.get_ui_instance($(v))
					if(widget){
						var db = find_widget(widget);
						if(db){
							if(db.uuid !== self.dashboard2.uuid) test.fail('found widget for different dashboard: ' + widget.uuid + ' ' + widget.options.title + ' ' + db.options.dashboard.title)
						}else{
							console.warn('widget not found in any dashboard', widget.options.title, v, widget)
							throw new Error('widget not found in any dashboard: "' + widget.options.title + '"')
						}
					}
				})

				setTimeout(function(){
					var n = $('body > .dashboard-widget:visible').length;
					console.log(' delayed visible count=', n, ' was', n1)
					n === n1 || console.log('changed - use async check instead!')
				}, 2000)

				return false;
			}

			delete self._inconsistency
			return true;
		}

		this.check = function(msg){
			return this.test.wait_for2(this.bind(this._check), msg || 'check')
		}

		this.ensure_widget = function(done){
			var self = this;

			if(this.first_widget().element.length) return done(this.first_widget().element);

			this.add_widget(function(a){
				var widget = $('body > .dashboard-widget:visible:first');
				self.test.assert(!widget.find('.chart-controls').length, 'dashboard widget should not have controls');
				self.test.assert(widget.length === 1, 'dashboard has widget');

				done(widget);
			});
		}

		this.ensure_editable_widget = function(done){
			var self = this;

			test.assert(this.dashboard2.active, 'not active')

			var found = false;
			this.dashboard2.widgets.all(function(w, c, r){
				if(w.instance && w.instance._chart && w.instance._chart.product){
					var product = w.instance._chart.product
					if(product.is_eccharts()){
						done($('#' + w.widget.name))
						found = true;
						return false;
					}
				}
			})

			if(!found){
				this.add_widget(function(_, widget){
					self.test.assert(typeof widget !== undefined, 'got widget');

					test.wait_for(
						function(){ return widget.element.find('.widget-head .ui-icon-pencil').length === 1 },
						'pencil not found'
					).then(function(){
						done(widget.element);
					});
				});
			}
		}

		this.ensure_interactive_chart = function(done){
			var self = this;
			var found = false;
			this.dashboard2.widgets.all(function(w, c, r){
				if(w.instance && w.instance.widgetName === 'chartDashboardWidget'){
					done(w.instance)
					found = true;
					return false;
				}
			})
			if(!found){
				console.log('adding interactive chart...');
				this.add_widget().then(function(widget){
					done(widget)
				})
			}
		}

		this.get_ui_instance = function(el){
			if(!el || !el.length) this.test.fail('get_ui_instance bad argument');
			var types = ['chart', 'basicChart', 'meteogram', 'controls']
			for(var i in types){
				var w = el.data('ui-' + types[i] + 'DashboardWidget') || el.data('ui-' + types[i] + 'DashboardWidget_10_days') || el.data('ui-' + types[i] + 'DashboardWidget_15_days') || el.data('ui-' + types[i] + 'DashboardWidget_efi') || el.data('ui-MeteogramDashboardWidget_extended') || el.data('uiMeteogramDashboardWidget_extended') || el.data('uiMeteogramDashboardWidget_vertical_all')
				if(w) return w;
			}
			console.warn('get_ui_instance: instance not found for element. data=', el.data());
		}

		/*
		 *  Add an eccharts widget.
		 */
		this.add_widget = function(done, name){
			console.log('%cadd_widget', 'background-color:#69b');
			var self = this;

			if(typeof done === 'string'){
				var tmp = done
				done = name
				name = tmp
			}

			return $.Deferred(function(df){
				self.check().then(function(){
					test.set_timeout(10000);

					var uuids = $('.dashboard-widget').map(function(i, v){ return self.get_ui_instance($(v)).uuid })

					var widget, wt;
					var n = self.n_widgets();
					var a = $.Deferred(function(d){
						bus.listen_once('dashboard-changed', function(){
							test.wait_for(
								function(){ return self.n_widgets() === n + 1},
								function(){ return 'n_widgets: ' + self.n_widgets() + ' != ' + (n + 1)}
							).then(function(){
								test.wait_for(function(){
									return $('.chart-image:visible').length; // TODO add more checking here
								}).then(a.resolve);
							})
						});
					})
					var b = $.Deferred(function(d){
						self.$dashboard.dashboard('add_widget', {widget: {kind: 'chart', data: {product: '2mT_wind30m'}}}, function(widget_name){
							var l = $('.dashboard-widget').map(function(i, v){
								if($(v).attr('id') === widget_name) return v;
							})
							if(l.length !== 1){
								$.each(l, function(i, v){ console.log('  ', v) });
							}
							test.assert(l.length === 1, 'added widget has duplicate id');
							widget = self.get_ui_instance($('#' + widget_name));
							if(widget){
								wt = new WidgetTestable(widget, self)
								test.assert(!~$.inArray(widget.uuid, uuids), 'instance is not new: uuid=' + widget.uuid);
								console.log('%cDashboardTestable: widget added:', 'color:#f00', widget_name, self.dashboard, widget);
								var img;
								test.wait_for(
									function(){
										img = widget.element.find('.chart-clip img')
										return !!img.length
									}, 'chart image'
								).then(function(){
									test.wait_for(
										function(){ return img.width() > 0 },
										'chart image empty'
									).then(function(){
// TODO use better value for expected image size - we get 59px when running under selenium
										test.assert(img.width() > 50, 'chart image too small: ' + img.width() + 'px');
										b.resolve()
									})
								})
							}else{
								console.error('widget added but data not found', $('#' + widget_name), $('#' + widget_name).data());
							}
						});
					});
					$.when(a.promise(), b.promise()).then(function(){
						function finished(){
							test._set_timeout_info('finished');
							done && done(null, widget);
							df.resolveWith(self.test, [widget]);
						}
						self.rename_widget(widget, test.name + ': ' + (name ? name : wt.title())).then(function(){
							finished()
						})
					})
				});
			}).promise();
		}

		this.remove_widget = function(widget_name){
			var self = this;
			if(!test.running) return;
			var widget = typeof widget_name === 'string' ? $('#' + widget_name) : $('body > .dashboard-widget:visible:first');
			var done = typeof widget_name === 'function' ? widget_name : null;
			test.assert(widget.length === 1);
			test._set_timeout_info('remove widget');

			console.log('%cremove_widget: name=', 'background-color:#fc8', widget_name, widget.attr('id'));
			var id = widget.attr('id');
			return $.Deferred(function(df){
				self.check().then(function(){
					self.test.set_timeout(10000);
					var _n_widgets = self.n_widgets();
					widget.find('.ui-icon-close').click().orelse(function(){ test.fail('cannot find widget'); });
					test.wait_for('.alert #_ok', 'ok button').then(function(){
						$('.alert #_ok').click();
						test.wait_for(
								function(){ return (self.n_widgets() === _n_widgets - 1); },
								function(){ return 'widget count: ' + self.n_widgets() + '!=' + (_n_widgets - 1) }
							)
							.then(function(){
								return test.wait_for(
									function(){ return $('#' + id).length === 0 },
									'widget dom not removed: #' + id
								)
							})
							.then(function(){
								self.check().then(function(){
									done && done();
									df.resolveWith(self.test);
								})
							})
					})
				})
			}).promise();
		}

		this.add_basic_widget = function(done){
			console.log('%cadd_widget (basic)', 'background-color:#69b');
			var self = this;
			self.test.set_timeout(10000);
			return $.Deferred(function(df){
				self.check().then(function(){
					var widget, wt;
					var n = self.n_widgets();
					var a = $.Deferred(function(d){
						bus.listen_once('dashboard-changed', function(){
							setTimeout(function(){
								test.assert(self.n_widgets() === n + 1, 'n_widgets: ' + self.n_widgets() + ' != ' + (n + 1));
								Test.wait_for(function(){
									return $('.chart-image:visible').length; // TODO add more checking here
								}).then(a.resolve);
							}, 1000);
						});
					});
					var b = $.Deferred(function(d){
						var proto = DashboardWidget.registry().basic_chart;
						self.$dashboard.dashboard('add_widget', {widget: proto}, function(widget_name){
							widget = self.get_ui_instance($('#' + widget_name));
							if(widget){
								wt = new WidgetTestable(widget, self)
								test.assert(widget.element.find('h3').text() === proto.title);
								console.log('%cDashboardTestable: widget added:', 'color:#f00', widget_name, self.dashboard, widget);
								b.resolve();
							}else{
								console.error('widget added but data not found', $('#' + widget_name), $('#' + widget_name).data());
							}
						});
					});
					$.when(a.promise(), b.promise()).then(function(){
						done && done(null, widget); df.resolveWith(self.test, [widget]);
					})
				});
			}).promise();
		}

		this.add_interactive_catalogue_widget = function(done){
			console.log('%cadd_widget (interactive)', 'background-color:#69b');
			var self = this;
			self.test.set_timeout(10000);
			return $.Deferred(function(df){
				self.check().then(function(){
					var wt;
					var widget;
					var n = self.n_widgets();
					var a = $.Deferred(function(d){
						bus.listen_once('dashboard-changed', function(){
							setTimeout(function(){
								test.assert(self.n_widgets() === n + 1, 'n_widgets: ' + self.n_widgets() + ' != ' + (n + 1));
								Test.wait_for(function(){
									return $('.chart-image:visible').length; // TODO add more checking here
								}).then(a.resolve);
							}, 1000);
						});
					});
					var b = $.Deferred(function(d){
						self.$dashboard.dashboard('add_widget', {widget: {kind: 'chart', data: {product: 's2s_anomaly_cawcr_2MTM'}}}, function(widget_name){
							widget = self.get_ui_instance($('#' + widget_name));
							wt = new WidgetTestable(widget, self)
							if(widget){
								console.log('%cDashboardTestable: widget added:', 'color:#0b0', widget_name, self.dashboard, widget);
								test.wait_for(function(){ return !!widget.element.find('img').attr('src') }, 'add_interactive_catalogue_widget: chart image').then(function(){
									b.resolve();
								})
							}else{
								console.error('widget added but data not found', $('#' + widget_name), $('#' + widget_name).data());
							}
						});
					});
					$.when(a.promise(), b.promise()).then(function(){
						done && done(null, widget);
						df.resolveWith(self.test, [widget]);
					})
				});
			}).promise();
		}

		this.add_meteogram = function(widget_type){
			widget_type = widget_type || 'meteogram_10_days';
			var self = this;

			return $.Deferred(function(df){
				// temporarily using an internal method (add_widget) until the menu is added to the page
				var proto = DashboardWidget.registry()[widget_type]
				test.assert(!!proto, 'no widget prototype for: ' + widget_type);
				self._add_widget_from_prototype(proto).then(df.resolve);
			}).promise();
		}

		this.add_controls = function(){
			if(false){
				// TODO why doesnt work?
				return this._add_widget_from_prototype(DashboardWidget.registry().controls)
			}else{
				var self = this;
				return $.Deferred(function(df){
					var proto = DashboardWidget.registry().controls
					test.assert(!!proto, 'no widget prototype for: controls');
					self._add_widget_from_prototype(proto).then(df.resolve);
				}).promise();
			}
		}

		this._add_widget_from_prototype = function(proto){
			var self = this;

			return $.Deferred(function(df){
				self.dashboard2.add_widget({widget: proto, top: true}, function(name){
					var widget = self.get_ui_instance($('#' + name))
					test.assert(!!widget, 'failed to find added widget. kind=' + proto.kind + ' name=' + name);
					test.assert(widget.element.find('h3').text() === proto.title, 'title');
					self.rename_widget(widget, test.name + ': ' + proto.title).then(function(){
						df.resolveWith(self.test, [widget]);
					})
				})
			}).promise();
		}

		this.rename_widget = function(widget, name){
			var self = this;
			test.assert(!!widget, 'rename_widget: widget=' + widget);
			test.assert(typeof name === 'string', 'rename_widget: name must be string');
			var set = $('.dashboard').dashboardSet('instance');
			var active = set.get_selected_dashboard2();
			test.assert(widget.options._dashboard.name === active.options.dashboard.name, 'widget not on active dashboard: active=' + ' widget=' + widget.options._dashboard.name)

			return $.Deferred(function(df){
				test.wait_for(
					function(){ return widget.element.is(':visible') },
					function(){ console.log('not visible widget:', widget); console.log('not visible widget:', widget.options.name, [widget.options.title]); console.log('not visible widget: dashboard=', widget.options._dashboard.title); console.log('not visible widget: element=', widget.element.is(':visible'), widget.element); return 'rename_widget: widget ' + widget.options.title + ' not visible' }
				).then(function(){
					test.assert(!widget.element.hasClass('editing'), 'rename_widget: already editing');

					var el = widget.element.find('h3');
					test.assert(el.length === 1, 'rename link not found for widget=' + widget.options.name);
					test.assert(!el.hasClass('editable-disabled'), 'rename_widget: editable disabled');
					el.click();
					test.wait_for('.editableform', 'editable: widget=' + widget.options.name + ' "' + widget.options.title + '"')
						.then(function(){
							var form = $('.editableform')
							form.find('input').val(name)
							test.wait_find(form, '.editable-submit').then(function(ok){
								ok.click()
								test.wait_for(
									function(){ return el.text() === name },
									function(){ return 'has new name: "' + el.text() + '" (expected "' + name + '")'}
								).then(function(){
									df.resolveWith(self.test);
								})
							})
						})
				})
			})
		}

		this.select = function(){
			var self = this;
			return $.Deferred(function(df){
				var tab = $('#dashboard-set .ui-tabs-nav a[href="#' + self.dashboard2.element.attr('id') + '"]')
				self.test.assert(tab.length === 1);
				self.dashboard2.bus.listen_once('tab-activated', function(){
					self.test.wait_for(function(){
						return tab.parent().hasClass('ui-tabs-active')
					}).then(function(){
						// TODO wait for all widgets to be displayed
						df.resolve()
					})
				}, undefined, true) // no PAST
				tab.click()
			})
		}

		this.preview = function(widget_name){
			var self = this;
			var done = typeof widget_name === 'function' ? widget_name : null;

			var contents = {
				chart: '.chart',
				meteogram: '.content'
			}

			return $.Deferred(function(df){
				test.wait_for(function(){
					return $('body > .ui-dialog-preview:visible').length === 0;
				}, 'init')
				.then(function(){ 
					$('#' + widget_name + ' .widget-content').click();
					test
						.wait_for($.map(contents, function(c){ return '#preview ' + c + ':visible'; }).join(', '), 'preview visible')
						.then(function(){
							test.assert($('body > .ui-dialog-preview .ui-dialog-titlebar-close').length > 0, 'have close button');
							test.wait_for(
								function(){ return $('body > .ui-dialog-preview .time-nav').length > 0},
								'have time navigator'
							).then(function(){
								test.assert($('body > .ui-dialog-preview .chart-controls > .bar-2').length == 1, 'single options menu');
								$('body > .ui-dialog-preview .ui-dialog-titlebar-close').click();
								test.wait_for(function(){
									return $('body > .ui-dialog-preview .content:visible').length === 0;
								}, 'preview closed')
								.then(function(){ done && done(); df.resolveWith(self.test); });
							})
						});
				});
			}).promise();
		}

		/*
		 * open/close the preview window for a widget multiple times
		 */
		this.multiple_previews = function(widget_name){
			var self = this;

			if(self.test.running) return $.Deferred(function(df){
				$.when(self.preview(widget_name)).then(function(){
					$.when(self.preview(widget_name)).then(function(){ df.resolveWith(self.test); });
				});
			}).promise();
		}

		this.edit_start = function(widget_name){
			var self = this;

			var selector = '#' + widget_name + ' .widget-head .ui-icon-pencil'
			test.assert($(selector).length === 1, 'widget button not found: ' + selector)

			return $.Deferred(function(df){
				$('#' + widget_name + ' .widget-head .ui-icon-pencil').click();
				test
					.wait_for(function(){
						return $('#' + widget_name + ' .edit .form:visible').length;
					})
					.then(function(){
						df.resolveWith(self.test);
					});
			}).promise();
		}

		this.edit_stop = function(widget_name){
			var self = this;

			var selector = '#' + widget_name + ' .widget-head .ui-icon-pencil'
			test.assert($(selector).length === 1, 'edit_stop: widget button not found: ' + selector) // note button is not visible because of hover

			return $.Deferred(function(df){
				$(selector).trigger('click')
				test
					.wait_for_remove('#' + widget_name + ' .edit .form', 'hide edit form')
					.then(function(){
						df.resolveWith(self.test);
					});
			}).promise();
		}

		this.rename = function(name){
			var self = this;
			test.assert(typeof name === 'string', 'tab rename: name not string');

			return $.Deferred(function(df){
				var el = $('.ui-tabs-nav a[href="#' + self.dashboard.name + '"]'); // TODO use set._nav_lookup
				test.assert(el.length === 1, 'tab link not found for name=' + self.dashboard.name);
				test.assert(el.text() !== name, 'tab rename: tab already named "' + name + '"')
				el.click();
				test.wait_for('.editableform', 'tab rename: editable-form')
					.then(function(){
						var form = $('.editableform')
						test.assert(form.find('input').length == 1, 'tab rename: input');
						form.find('input').val(name)
						form.find('.editable-submit').click()
						test.wait_for(
							function(){ return el.text() === name },
							function(){ return 'tab rename: has new name: expected: "' + name + '" got: "' + el.text() + '"'})
						.then(function(){
							test.wait_for(
								function(){ console.log('       d=', self.dashboard2); return self.title() === name; },
								function(){ return 'internal rename "' + self.title() + '" != "' + name + '"'})
							.then(function(){
								df.resolveWith(self.test);
							})
						})
					})
			}).promise();
		}

		this.set_n_columns = function(n){
			console.log('%cset_n_columns', 'color:#0a0', n);
			var self = this;
			if(n < 1 || n > 6) return test.fail('bad columns number:', n);

			var dashboard = $('.dashboard').dashboardSet('get_selected_dashboard2');
			var n_widgets = this.n_widgets();

			function check_n_columns(n){
				if(dashboard.options.dashboard.columns.length !== n) return false;
				if(dashboard.widgets.columns.length !== n) return false;
				return true;
			}

			function menu_item_is_selected(n){
				// n is zero indexed
				return $('.layout .menu-radio :checked').parent().index() === n;
			}

			function widget_size(){
				return $.Deferred(function(df){
					self.iter(function(w, c, r, next){
						// TODO wait for the widget to change size
						next()
					}, function(){
						df.resolveWith(self.test);
					})
				}).promise();
			}

			return $.Deferred(function(df){
				self.check().then(function() {
					test.set_timeout(10000);
					var menu = $('#app-menu #main-menu');
					test.assert(menu, 'app menu')
					var layout_menu = menu.find('> li:eq(1)');
					test.assert(layout_menu, 'layout menu')
					test.assert(layout_menu.find('a:first'));
					layout_menu.find('a:first').click();
					test
						.wait_for('#app-menu #main-menu > li:eq(1) .btn-group.open', 'menu open')
						.then(function(){
							return $.Deferred(function(df2){
								var dropdown = '#app-menu #main-menu > li:eq(1) .dropdown-menu';
								var menu_item = dropdown + ' li a[href="#radio-' + n + '"]';
								test.wait_for(menu_item, 'menu item ' + n).then(function(){
									test.assert($(dropdown + ' > li').length === 6, '6 menu items')
									test.assert(!menu_item_is_selected(n - 1)); // click will not work if already set to n.
									$(menu_item).click();
									test.wait_for_remove('#menu-panes .layout .menu-list', 'menu close')
										.then(function(){
											test.assert(check_n_columns(n), 'incorrect number of columns');
											test.wait_for(function(){ return self.n_widgets() === n_widgets; }, 'incorrect number of widgets');
											test.wait_for(function(){ return menu_item_is_selected(n - 1); }, 'correct menu item not selected');
											test.wait_for(widget_size).then(function(){
												df2.resolveWith(self.test);
											})
										});
								})
							}).promise();
						})
						.then(function(){
							df.resolveWith(self.test);
						});
				})
			}).promise();
		}

		// TODO use this.dashboard2.widgets.find() instead
		this.find_widget_by_name = function(name){
			var widget;
			this.dashboard2.widgets.all(function(v, c, r){
				if(v.widget.name === name) widget = v.instance;
			})
			return widget
		}

		this.validate = function(n){
			// TODO see dst.validate_active_dashboard
			// see also this.check
		}

		this.bind = function(fn) {
			var self = this;
			return function() {
				return fn.apply(self, arguments);
			};
		}
	}

	return DashboardTestable;
});
