/*
 *
 */
define(['jquery', 'virtual-dom/h', 'draggable', './index.css', 'bootstrap_modal/bootstrap_modal.css'], function($, h, Draggable) {

var zIndex = 80;

function Dialogue(options){
	var self = this;

	var o = this.options = $.extend({
		title: '',
		id: undefined, // mandatory
		size: undefined,
		position: undefined,
		footer: $.noop,
		resizable: false,
		draggable: false, // if set, use whole dialogue as drag handle
		on_close: $.noop
	}, options);

	this.size = this.options.size
	this.min_size = {width: 320, height: 320}
	this.position = options.position || {top: 100, left: 100}
	this._open = true;

	o.renderer.on.render.once(function(){
		self.el = document.getElementById(o.id)
		self.header = self.el.querySelector('.modal-header')

		new Draggable(o.draggable ? self.el : self.header, {
			setPosition: false,
			filterTarget: function(a) {
				return a.nodeName !== 'INPUT';
			}
		})

		var $resize = $(self.el).find('.resize')
		var resize_down, resize_timer;
		var offset2;
		var s;
		$resize
			.on('mousedown', function(e) {
				if(e.which && e.which !== 1) return;

				self.mouse.offset = $resize.offset();
				offset2 = $(self.el).offset();
				s = {x: e.pageX - self.mouse.offset.left, y: e.pageY - self.mouse.offset.top}
				resize_down = true;

				$(document)
					.on('mousemove.dialogue', move)
					.on('mouseup.dialogue', up)

				resize_timer = resize_timer || setInterval(function(){ self.options.on_resize(self.size) }, 2000)

				e.preventDefault();
			})
			.on('mouseup', up)
			.on('mousemove', move)

		function move(e) {
			if(resize_down){
				var x = e.pageX - offset2.left;
				var y = e.pageY - offset2.top;
				$(self.el).css((self.size = {width: Math.max(self.min_size.width, x + s.x), height: Math.max(self.min_size.height, y + s.y)}))

				e.preventDefault();
			}
		}

		function up() {
			if(resize_down){
				resize_down = false
				resize_timer = clearInterval(resize_timer)
				$(document).off('.dialogue')
				self.options.on_resize(self.size)
			}
		}
	})
}

Dialogue.prototype = {

	render: function(body) {
		var self = this, o = this.options;

		var css = {};
		for(var p in this.size) css[p] = this.size[p] + 'px'
		for(var p in this.position) css[p] = this.position[p] + 'px'

		var footer = o.footer()
		if(footer) footer = h('.modal-footer', footer)

		return h('.dialogue#' + o.id + (this._open ? '' : '.closed') + (o.className ? '.' + o.className : ''), {style: css}, [
			h('.modal-header', [
				h('h2', {
					/*
					onmousedown: function(e){
						var mouse = self.mouse
						if(e.which && e.which !== 1) return;

						mouse.down = true
						var rect = self.header.getBoundingClientRect();
						mouse.offset = {left: rect.left, top: rect.top};
						mouse.start = {x: e.pageX - self.mouse.offset.left, y: e.pageY - self.mouse.offset.top}

						$(document)
							.on('mousemove.dialogue', function(){ return mouse.move.apply(self, arguments)})
							.on('mouseup.dialogue', function(){ return mouse.up.apply(self, arguments)})

						self.move_to_top();

						e.preventDefault();
					},
					onmousemove: function(e){
						if(self.mouse.down){
							self.mouse.move.apply(self, arguments)
						}
					},
					onmouseup: function(){
						self.mouse.up.apply(self)
					}
					*/
				}, [
					o.title,
					h('.close', {
						onclick: function(){
							console.log('close');
							self.close();
						}
					})
				])
			]),
			body,
			footer,
			o.resizable ? h('.resize.ui-icon.ui-icon-grip-diagonal-se') : undefined
		])
	},

	open: function() {
		this._open = true;
		this.options.renderer.render();
	},

	close: function() {
		this._open = false;
		this.options.on_close()
		this.options.renderer.render();
	},

	mouse: {
		down: false, // warning, although it might be safe, these shouldnt really be in the prototype
		start: undefined,
		offset: undefined,
		pos: undefined,

		/*
		move: function(e) {
			var mouse = this.mouse;

			var x = e.pageX - mouse.offset.left;
			var y = e.pageY - mouse.offset.top;
			var dx = x - mouse.start.x;
			var dy = y - mouse.start.y;
			var size = self.size || {width: 10, height: 10}
			$('#' + this.options.id).css((mouse.pos = {
				left: Math.min(Math.max(this.position.left + dx, 10 - size.width), $(window).width() - 20),
				top: Math.min(Math.max(this.position.top + dy, 10 - size.height), $(window).height() - 20)
			}))

			e.stopPropagation();
			e.preventDefault();
		},

		up: function() {
			var mouse = this.mouse
			if(mouse.down){
				mouse.down = false
				if(mouse.pos){
					this.position = mouse.pos
					mouse.pos = null
				}
				$(document).off('.dialogue')
			}
		}
		*/
	},

	move_to_top: function() {
		var group = $.makeArray($('body > .dialogue')).sort(function (a, b) {
			return (parseInt($(a).css('zIndex'), 10) || 0) - (parseInt($(b).css('zIndex'), 10) || 0);
		});

		if(group.length) {
			$(group).each(function(i) {
				this.style.zIndex = zIndex + i;
			});

			$(self.el).css({zIndex: zIndex + group.length});
		}
	}
}

return Dialogue;

});
