import Settings from 'settings'
import eccharts from 'eccharts-2'
import loc from 'chart/model/location'

import Style from 'ol/style/style'
import Circle from 'ol/style/circle'
import Fill from 'ol/style/fill'
import Stroke from 'ol/style/stroke'
import Feature from 'ol/feature'
import Point from 'ol/geom/point'
import Proj from 'ol/proj'
import VectorLayer from 'ol/layer/vector'
import VectorSource from 'ol/source/vector'
import Modify from 'ol/interaction/modify'

function Probe() {
	var self = this;

	var feature = new Feature({geometry: new Point([loc.value.lon, loc.value.lat])});

	feature.setStyle(new Style({
		image: new Circle({
			radius: 10,
			fill: new Fill({
				color: [255, 204, 102, 1]
			}),
			stroke: new Stroke({
				color: [255, 255, 255, 1],
				width: 1.5
			})
		}),
		zIndex: 1
	}))

	feature.getGeometry().on('change', function(e) {
		var coords = this.getCoordinates()
		loc.set({lat: coords[1], lon: coords[0]})
	})

	var map = eccharts.chart().map

	var source = new VectorSource({features: [feature]})
	var layer = new VectorLayer({source: source})

	var modify = new Modify({source: source}) // enable probe dragging
	map.addInteraction(modify);

	map.getLayers().push(layer)
}

Probe.prototype = {
	render: function() {
	}
}

export default Probe
