import Bus from 'bus'
import Time from 'time'
import h from 'virtual-dom/h'
import mixins from 'mixins'
import Button from 'grid/button'
import ToggleButton from 'grid/togglebutton'
import Slider from './canvas_slider'

var primary_axis_name   = 'valid_time';
var secondary_axis_name = 'base_time';

function TimeNavigator(args) {
	var self = this

	var o = this.options = extend({
		time: null,     // type Observable
		animator: null, // type ChartAnimator
		bus: Bus()
	}, args);

	this.buttons = [[
		new Button({
			icon: 'ui-icon-carat-1-w',
			className: 'prev-step',
			on_click: function() {
				console.log('prev.click')
				o.time.previous()
			}
		}),
		new Button({
			icon: 'ui-icon-carat-1-e',
			className: 'next-step',
			on_click: function() {
				console.log('next.click')
				o.time.next()
			}
		})],
		[
		new ToggleButton({
			icons: ['ui-icon-play', 'ui-icon-pause'],
			className: 'animate',
			on: function() {
				console.log('animate start')
				o.animator.toggle()
			},
			off: function() {
				console.log('animate stop')
				o.animator.toggle()
			},
			renderer: o.renderer
		})
		]
	]

	this.slider = new Slider({
		height: 24,
		renderer: o.renderer,
		on_select: this.bind(this._send_time_change_from_date),
		range_selection: {
			enabled: true,
			on_select: this.bind(self._send_animation_selection_change)
		}
	})

	o.bus.listen_once(self, 'chart-product-availability', function(){
	});

	o.bus.listen(self, 'chart-product-availability', this.bind(this._update));

	o.time.subscribe(function(time) {
		self.slider.setVal(time[self.primary_axis_name])
		self.current_idx = time.index(self.dates)
		self.update_button_states()
	})
}

TimeNavigator.prototype = extend({
	render: function() {
		return h('.time-navigator', [
			h('.time-nav', [
				h('.buttons', this.buttons[0].map(function(b){ return b.render() })),
				h('span', 'VT:'),
				this.slider.render(),
				this.buttons[1][0].render()
			])
		])
	},

	resize: function(width) {
		this.slider.size(width)
	},

	_update: function(_, msg) {
		var self = this, o = self.options;

		function validate(msg){
			var axis_values = msg[primary_axis_name].values
			if(!(axis_values[0] instanceof Time)) return false;
			var refp;
			for(var i in axis_values){
				var date = axis_values[i].valid_time;
				if(refp && ((date - refp) < -0)){
					setTimeout(function(){console.error('TimeNavigator.verify_dates: valid_time (' + i + ') out of range! ' + date.format('mmmdd HH:MM') + ' should be later than ' + (refp && refp.format('mmmdd HH:MM')))}, 1000);
					return false;
				}
				refp = date;
			}

			refp = null;
			axis_values = msg[secondary_axis_name].values;
			//axis_values.length || console.warn('TimeNavigator.verify_dates: no secondary axis dates');
			for(i in axis_values){
				date = axis_values[i].base_time;
				if(refp && ((date - refp) < -0)){
					(function(){
						setTimeout(function(){$.logWarning('TimeNavigator.verify_dates: ' + secondary_axis_name +' (' + i + ') out of range! ' + date.format('mmmdd HH:MM') + ' should be later than ' + refp.format('mmmdd HH:MM'))}, 1000);
					})();
					return false;
				}
				refp = date;
			}
			return true;
		}

		if(__DEV__ && !validate(msg)) return;

		this.last_msg = msg;

		this.primary_axis_name   = this.flipped ? secondary_axis_name : primary_axis_name;
		var _secondary_axis_name = this.flipped ? primary_axis_name   : secondary_axis_name;

		this.dates       = msg[this.primary_axis_name].values;
		this.current_idx = msg[this.primary_axis_name].index;
		this.ref_date    = msg[this.primary_axis_name].values[0].base_time;

		this.primary = {
			axis       : msg[this.primary_axis_name]
		}

		this.secondary = {
			axis_name  : _secondary_axis_name,
			axis       : msg[_secondary_axis_name],
			dates      : msg[_secondary_axis_name].values,
			index      : msg[_secondary_axis_name].index
		}

		this._set_selectlist();
		/* TODO
		(this.secondary.dates.length && o.show_bt) || (this.selector.hide(), this.secondary_label.hide());
		*/

		if(self.dates.length > 1)
			this._enable();
		else
			this._disable();

		self.slider.setDates({
			dates: (self.dates)
				? self.dates.map(function(a) { return a[self.primary_axis_name]})
				: null,
			index: self.current_idx,
			ref: self.ref_date,
			format: msg.format
		});
	},

	_enable: function() {
	},

	_disable: function() {
	},

	update_button_states: function(){
		var self = this;

		if(self.primary_axis_name){
			var length = self.last_msg[self.primary_axis_name].values.length;
			/* TODO
			self.b2.toolbar((self.current_idx === 0) ? 'disable' : 'enable', 'prev-step');
			self.b2.toolbar((self.current_idx === length - 1) ? 'disable' : 'enable', 'next-step');
			*/
		}
	},

	_set_selectlist: function() {
		var self = this;

		var primary_dates = this.primary.axis.values.map(function(a) { return a[self.primary_axis_name] });
		var secondary_dates = this.secondary.dates.map(function(a) { return a[self.secondary.axis_name] });
		var not_latest = (
			this.flipped ?
				(this.primary.axis.index != primary_dates.length - 1)
				: (this.secondary.index != secondary_dates.length - 1)
		);

		/* TODO
		this.selector.dateSelect('setDates', secondary_dates, secondary_dates[this.secondary.index], (false && not_latest && !this.flipped));

		self.selector.find('.button div').pulsate(not_latest);
		*/

		this.is_latest = !not_latest;
	},

	_send_time_change_from_date: function(date){
		var t = this._lookup_date(date);
		if(t) this.options.time.set(t);
		else console.warn('TimeNavigator._send_time_change_from_date: lookup failed. date=', date, ' axis=' + axis_name);
	},

	_send_animation_selection_change: function(selection){
	},

	_lookup_date: function(time, axis_name){
		if(!time) return;
		if(!axis_name) axis_name = this.primary_axis_name;

		var dates = this.last_msg[axis_name].values;

		for(var i in dates){
			var item = dates[i][axis_name];
			if(item.getTime() === time.getTime()){
				return dates[i];
			}
		}
	}
}, mixins)

export default TimeNavigator
