import h from 'virtual-dom/h'
import loc from 'chart/model/location'
import Unit from 'unit/unit'
import Ajaxqueue from 'ajax_queue'
import Bus from 'bus'
import Layer from 'chart/layer'
import mixins from 'mixins'
import eccharts from 'eccharts-2'
import './probe.css'

var chart = eccharts.chart()
var time = eccharts.time

function Probe(args) {
	var self = this;
	var o = this.options = extend({}, args)

	this.probes = []

	loc.subscribe(this.bind(this.fetch))
	time.subscribe(this.bind(this.fetch))

	Bus().listen('chart-content-update', function() {
		self.fetch()
	})
}

Probe.prototype = extend({
	render: function() {
		return h('.body', [h('div', [
			h('.title', ['Data values near location '].concat(
				Unit.renderGeoLocation(loc.value),
				[', ' + time.value.asText()],
				this.title && ((this.title.length > 30 ? h('br') : ', ') + this.title)
			)),
			h('ul', this.probes.map(function(probe){
				if(probe.values && probe.values.length){
					var p = probe.values[0] // Ignore mult-field replies
					var best = p.neighbours[p.selected]

					return h('li.row', [
						h('.cell', probe.title),
						h('.cell', best.values
							? [new Unit.Unit(probe.units[0]).render(best.values[0]), ' ', new Unit.Unit(probe.units[1]).render(best.values[1])]
							: new Unit.Unit(probe.unit).render(best.value)
						),
						h('.cell.c', p.method),
						h('.cell', Unit.renderGeoLocation(best)),
						h('.cell.r', new Unit.Unit(best.distance_unit).asHTML(best.distance))
					])
				}
			}))
		])])
	},

	fetch: function() {
		var self = this;

		Ajaxqueue({
			url: '/backend/probe/',
			name: 'probe',
			cache: 10 * 60,
			cancelled_by: chart.product.owner,
			json: {product: chart.product.as_json(null, time.value), location: loc.value},
			success: function(result){
				self.title = result.title
				self.probes = result.probe.reverse()

				self.options.renderer.render()
			}
		})
	}
}, mixins)

export default Probe
