// see also vform
import extend from 'extend'
import h from 'virtual-dom/h'
import Unit from 'unit'
import CityAutocomplete from 'city-autocomplete/2' // TODO load dynamically?

function Location(options) {
	var self = this

	var o = this.options = extend({
		value: {}
	}, options)

	this._value = extend({}, o.value);

	this.autocomplete = new CityAutocomplete({
		id: 'cities',
		value: this._value.station_name,
		select: function(pt) {
			self._value = extend({}, self._value, {
				lat: Unit.formatLatitude(pt.lat, true),
				lon: Unit.formatLongitude(pt.lon, true),
				alt: pt.altitude,
				station_name: pt.title
			})
			o.renderer.render()
		},
		renderer: o.renderer
	})
}

Location.prototype = {
	render: function() {
		var self = this, o = this.options

		return h('.form-location', [
			h('label', 'Latitude'),
			h('input.form-text.lat', {type: 'text', value: Unit.formatLatitude(this._value.lat, true), onchange: function(e){
				self._value.lat = e.target.value
			}}),
			h('label', 'Longitude'),
			h('input.form-text.lon', {type: 'text', value: Unit.formatLongitude(this._value.lon, true), onchange: function(e){
				self._value.lon = e.target.value
			}}),
			h('label', 'Altitude'),
			h('input.form-text', {type: 'text', disabled: true}),
			h('label', 'City'),
			h('#cities', this.autocomplete.render()),
			h('p.location-help', 'Please note that when using the city finder the lat/lon will not be updated unless an item is selected from the drop-down list, otherwise you can use the text purely as a label.')
		])
	},

	collect: function() {
		return this.options.value = {
			lat: Unit.parseLL(this._value.lat),
			lon: Unit.parseLL(this._value.lon),
			// TODO use either city or station_name, not both
			city: this._value.station_name,
			station_name: this._value.station_name,
			altitude: this._value.alt
		}
	},

	cancel: function() {
		this._value = this.options.value;
		this.autocomplete.options.value = this._value.station_name
	}
}

function Country(options) {
	var self = this

	var o = this.options = extend({
		value: {}
	}, options)

	this.autocomplete = new CityAutocomplete({
		id: 'countries',
		value: '',
		is_country: true,
		select: function(country) {
			self._value = country
			o.renderer.render()
		},
		renderer: o.renderer
	})
}

Country.prototype = {
	render: function() {
		return h('.form-location', [
			h('.country-location-help', [
				h('b', 'Please enter a country name'),
				h('br'),
				'You will then be able to select a location in the left column.'
			]),
			h('#countries', this.autocomplete.render()),
		])
	},

	collect: function() {
		return this.options.value = this._value
	}
}

function Form(options) {
	var o = this.options = extend({
		items: []
	}, options)
}

Form.prototype = {
	render: function() {
		return h('.form', this.options.items.map(function(item){ return item.render() }))
	},

	collect: function() {
		return this.options.items.map(function(item){ return item.collect() })
	},
}

export {Form, Location, Country}
