/*
 *  Eccharts layer list
 *
 */
import RenderTree from 'render-tree'
import h from 'virtual-dom/h'
import Bus from 'bus'
import Layer from 'chart/layer'
import Button from 'grid/button'
import Sortable from 'sortablejs'
import eccharts from 'eccharts-2'
import Form from 'vform'
import 'vform/handlers/thumbnail'

import 'layer-list/2.css'

function LayerList(options) {
	var self = this;

	var o = this.options = extend({
		bus: Bus()
	}, options);

	this.bus = o.bus
	this.items = []

	o.renderer.on.render.once(function() {
		var el = document.querySelector('.layer-list .items'); // TODO should search relative to widget root
		var sortable = Sortable.create(el, {
			onSort: function (e) {
				// TODO hide any layer edit forms

				self.bus.send('layer-order-change', {order: sortable.toArray().reverse()});
			}
		})
	})

	this.add_layers = new Button({
		icon: 'ui-icon-circle-plus',
		text: 'Add layers ...',
		show_text: true,
		icon_position: 'left', // TODO
		className: 'add-layer',
		on_click: function() {
			self._open_thumbnail_dialogue();
		}
	})

	o.bus
		.listen('chart-content-update', function(event, msg) {
			self.product = msg.product
			self.time = msg.time

			function add_item(i, layer) {

				var button = new Button({
					renderer: o.renderer,
					icon: 'ui-icon-trash',
					className: 'remove',
					on_click: function(e) {
						var layers = self.product.layerList()
						if(layers.length > 1) {
							button.disable()
							o.bus.send('layer-delete', {layer_id: layers[i].id });
						}
					}
				})

				self.items.push({
					remove: button,
					edit: new Button({
						renderer: o.renderer,
						icon: 'ui-icon-pencil',
						className: 'edit',
						on_click: function(e) {
							console.log('edit click this=', this);
							console.log('edit click e=', e);
							console.log('edit click i=', i, self.editing && self.editing.i);
							if(self.editing && self.editing.i === i){
								self.editing = null
							}else{
								self.editing = {
									i: i,
									form: new Form(self._make_form(layer))
								}
							}
							o.renderer.render()
						}
					})
				})
			}

			// Ensure the items array is the correct length
			// - There is the possibility that self.items gets out of sync
			//   with product.layers - it may be neccesary to rebuild
			//   self.items from scratch here
			var layers = self.product.layerList()
			var n_layers = layers.length;
			if(layers.length < self.items.length){
				// if the delete button is disabled, this indicates that it might be the deleted layer
				var disabled = self.items.map(function(l, i){
					if(l.remove.disabled){
						return i;
					}
				})
				disabled.forEach(function(l, i){
					self.items.splice(l, 1)
				})
			}
			self.items = self.items.splice(0, layers.length);
			if(self.items.length < n_layers){
				for(var i=self.items.length;i<n_layers;i++){
					add_item(i, layers[i])
				}
			}

			self.render();
		})
}

LayerList.prototype = {
	render: function(body) {
		var self = this;

		self.product && self.product.layerList().forEach(function(layer, i) {
			self.product.layer_icon(layer.id, null, self.time, function(result) {
				var url = result && result.url;
				var changed = url !== self.items[i].thumb
				if(changed){
					self.items[i].thumb = url
					self.options.renderer.render();
				}
			})
		})
		return this._render()
	},

	_render: function(body) {
		var self = this;

		var items = []
		this.product && this.product.layerList().forEach(function(layer, i) {
			var item = self.items[i]
			items.unshift(h('div', {attributes: {'data-id': layer.id}}, [
				h('img.thumb', {src: self.items[i].thumb}),
				h('span.title', layer.title),
				item.edit.render(),
				item.remove.render()
			]))
		})

		if(self.editing){
			var layers = self.product.layerList()
			items.splice(layers.length - self.editing.i, 0, self.editing.form.render())
		}

		return h('.layer-list' + (self.editing ? '.editing' : ''), [
			this.add_layers.render(),
			h('.items', items)
		])
	},

	_make_form: function(layer) {
		var self = this;

		var vars = layer.variables || [];
		var styles = layer.styles || [];

		var items = [];
		vars.forEach(function(val, i) {
			items.push(val);
		});

		var names = [];
		var labels = {};
		styles.forEach(function(style, j) {
			names.push(style.name);

			labels[style.name] = {
				url: '//' + ECMWF.options.backend_host + '/backend/get-style-thumbnail/' + style.name + '/',
				title: style.title
			}
		});

		names.length > 1 && items.push({
			name: 'style',
			title: 'Style',
			type: 'thumbnail',
			value: layer.style,
			values: names,
			labels: labels,
		})

		return {
			el: '.layer-list',
			items: items,
			values: layer.values || {},
			show_cancel: false,
			ok_label: 'Apply',
			ok: function(result) {
				self.bus.send('layer-options-change', {layer_id: layer.id, options: result});
				self.editing = null
				self.options.renderer.render()
			}
		}
	},

	_open_thumbnail_dialogue: function() {
		require(['thumbnail-dialogue/2'], function(LayerSelector){
			self.dialogue = self.dialogue
				? self.dialogue.open()
				: new LayerSelector({kind: 'layer', hide_user_items: false, select: function(selection){
					console.log('LayerList got add:', selection.name());
					eccharts.chart().add_layer(selection)
				}})
		})
	}
}

export default LayerList;
