import $ from 'jquery'
import 'test-lib'
import GridTestable from 'grid/testable'

export default function MeteogramsTestable(test) {
	this.test = test;
	this.chart = $('#chart');
	var self = this;

	this.count = function(){
		return $('#meteograms .content li').length;
	}

	this.ensure_meteogram = function(){
		test._set_timeout_info('ensure_meteogram');
		return new Promise(function(resolve){
			if(self.count()) resolve();
			else self.add_meteogram().then(resolve)
		})
	}

	this.add_meteogram = function(){
		test._set_timeout_info('add_meteogram');
		return new Promise(function(resolve){
			self.open_selector().then(function(){
				self.selector_select().then(function(){
					self.close_selector().then(resolve)
				});
			})
		})
	}

	this.remove_meteogram = function(n){
		var test = this.test;
		test._set_timeout_info('remove_meteogram');

		n = n || 0

		return new Promise(function(resolve){
			test.set_timeout(15000);
			test.assert(self.count() > 0, 'cannot remove meteogram. none to remove');
			var content = $('#meteograms .content');
			var item = content.find('li:eq(' + n + ')');
			test.assert((item.length === 1), 'cannot find item');
			var count = self.count()//meteograms.length

			// because the tools use hoverIntent, we cannot use a simple event to open them
			item.find('.meteogram').thumbnailTools('show', true);
			var button = item.find('.tools .ui-icon-trash')
			test.assert((button.length > 0), 'cannot find button');
			/*
			var title = item.find('.meteogram-title span').text();
			console.log('title=', title);
			test.assert((title.length > 0), 'meteogram has no title');
			if(!title.length) return;
			*/
			button.click();

			test.wait_for(function(){
				return self.count() === count - 1;
			}, 'remove count').then(resolve);
		})
	}

	this.open_selector = function(){
		test._set_timeout_info('open_selector');
		return new Promise(function(resolve){
			$('#meteograms .more a:last').click();
			test.wait_for('#meteogram-select:visible').then(function(){
				test.wait_for('#meteogram-select .catalogue-grid .item').then(function(){
					resolve()
				}, 'selector items');
			}, 'selector open');
		})
	}

	this.close_selector = function(){
		return new Promise(function(resolve){
			$('#meteogram-select .close').click();
			setTimeout(function(){
				resolve()
			}, 1000)
		})
	}

	this.selector_search = function(search){
		return new Promise(function(resolve){
			var gt
			(gt = new GridTestable(test, '#meteogram-select')).ready.then(function(){
				gt.search(search).then(resolve)
			})
		})
	}

	this.selector_select = function(search){
		test._set_timeout_info('selector_select');

		return new Promise(function(resolve){
			var n_meteograms = $('#meteograms .content > ul > li').length;
			console.log('>>> clicking...');
			Test.click($('#meteogram-select .catalogue-grid .item img:first'));
			//we could also check for this message: 'multigrams-list-changed'
			test.wait_for(function(){
					return $('#meteograms .content > ul > li').length == (n_meteograms + 1);
				}, 'n_items + 1')
				.then(resolve)
		})
	}

	this.add_efi = function (Xn){
		return new Promise(function(resolve){
			self.open_selector().then(function(){
				self.selector_search('efi').then(function(product){
					self.selector_select().then(function(){
						test.wait_for('#meteograms .meteogram-title:last:contains(' + product.title_() + ')', function(){ return 'title: got "' + $('#meteograms .meteogram-title:last').text() + '"'}).then(function(){
							self.close_selector().then(function(){
								resolve()
							})
						})
					})
				})
			})
		})
	}

	this.remove_efi = function (){
		test._set_timeout_info('remove_efi');
		var n_items = $('#meteograms .content li h5:contains("EFI")').length;
		test.assert(n_items > 0, 'remove_efi: no efi\'s to remove?')
		return this.remove_meteogram(n_items - 1)
	}

	this.check_help = function(){
		return new Promise(function(resolve){
			$('#meteograms .meteogram:first').thumbnailTools('show'); // triggering mouseenter doesnt work because of hoverIntent
			var panel = 'body > .help-panel';
			test.assert_no_element(panel);
			var button = '#meteograms .meteogram:first .ui-icon-help';
			test.assert(button, 'check_help: help button');
			$(button).click()
			test.wait_for(panel, 'help-panel').then(function(){
				test.wait_for(panel + ' .content p', 'help-panel content').then(function(){
					$(panel + ' .ui-icon-closethick').click()
					test.wait_for_remove(panel).then(function(){
						resolve();
					})
				})
			})
		})
	}

	this.bind = function(fn) {
		var self = this;
		return function() {
			return fn.apply(self, arguments);
		};
	}
}
