define(['config', 'jquery', 'unit', 'bus', 'log', 'ajax_queue', 'settings', 'chart/model/location', 'chart/layer', './probe.css'], function(ECMWF, $, Units, Bus, Log, Ajaxqueue, Settings, loc, Layer){
	loc = loc.default
	Log = Log.default

    var count = 0;

    var open_probes = [];

    $.openProbe = function(o) {
        open_probes = $.grep(open_probes, function(x) { return x != o; });
        open_probes.push(o);
        Bus().send('visible-probe-windows', {count: open_probes.length});
    }

    $.closeProbe = function(o) {
        open_probes = $.grep(open_probes, function(x) { return x != o; });
        Bus().send('visible-probe-windows', {count: open_probes.length});
    }

    loc.subscribe(function(val) {
        if ((open_probes.length === 0) && (val.from === 'chart')) {
            Bus().send('do-menu-views-probe-change', { value: true })
        }
    })

    var Probe = {

        options: {
            messageBus: Bus()
        },

        _init: function(){
            var self = this, o = this.options;

            count++;

            self.name = 'probe_' + count;

            self.element.addClass('probe');

            self.loc = self.element.div('.location');
            self.content = self.element.div('.content');

            self.n_pending_requests = 0;

            self.element
                .on('show', function(){
                    $.openProbe(self);
                    self._update();
                })
                .on('hide', function(){
                    $.closeProbe(self);
                });

            self._update = $.throttle(300, function() {
                self.__update();
            });

            o.location.subscribe(function(msg){
                self._update();
            })

            o.messageBus
                .listen('chart-time-select', function(event, msg){
                    self.time = msg;
                    self._update();
                })
                .listen('chart-content-update', function(event, msg){
                    self.product = msg.product;
                    self.time = msg.time || self.time;
                    self._update();
                })
                .listen('chart-animation-start', function() {
                    self.animating = true;
                })
                .listen('chart-animation-stop', function() {
                    self.element.css({minWidth: ''});
                    self.animating = false;
                });

            setTimeout(function(){ self._update(); }, 400);
        },

        _reset: function(txt){
            var self = this;

            self.content.empty();
            txt && self.loc.html(txt) || self.loc.empty().div('Click on the map with the probe tool:').parent().ul('.ec-toolbar').li('.sprite-outer').div('.button select sprite');
        },

        __update: function(){
            var self = this, loc = this.options.location.value;

            if (!loc || !self.product || !self.time || !(self.element.is(":visible"))) {
                self._reset();
                self.element.resize();
                return;
            }

            self.element.trigger('loading');

            var title = 'Getting values for location ' + Units.formatGeoLocation(loc) + '...';
            if (loc.title) {
                title += ', ' + loc.title;
            }
            self.loc.html(title);

            self.n_pending_requests++;

            Ajaxqueue({
                url: '/backend/probe/',
                name: self.name,
                cache: 10*60,
                cancelled_by: self.product.owner,
                statistics: true,
                json: { product : self.product.as_json(null, self.time), location: loc},
                success: function(json){
                    self.n_pending_requests--;

                    var title = 'Data values near location ' + Units.formatGeoLocation(json);
                    title += ', ' + self.time.asText();
                    if (json.title) {
                        var br = json.title.length > 30 ? '<br/>' : ', ';
                        title += br + json.title;
                    }

                    self.element.trigger('loading', 'false');
                    self.loc.html(title);
                    var table = self.content.empty().table({headers: ['Layer', 'Value', 'Point selected', 'Location', 'Distance']});

                    table.find('th').each(function(i){if(!(i % 2)) $(this).addClass('even')});

                    //var layers = self.product.layerList();

                    $.each(json.probe.reverse(), function(i, probe){

                        if (Array.isArray(probe) || !probe.values) {
                            return;
                        }

                        var p = probe.values[0]; // Ignore mult-field replies

                        var best = p.neighbours[p.selected];

                        var tr = table.tr();

                        tr.td().html(new Layer(self.product.layerByID(probe.id)).get_title());
                        tr.td('.r').html(
                            best.values
                                ? new Units.Unit(probe.units[0]).asHTML(best.values[0]) + ' ' + new Units.Unit(probe.units[1]).asHTML(best.values[1])
                                : new Units.Unit(probe.unit).asHTML(best.value)
                        );
                        tr.td('.c').html(p.method);
                        tr.td().html(Units.formatGeoLocation(best));
                        tr.td('.r').html(new Units.Unit(best.distance_unit).asHTML(best.distance));

                        tr.find('td').each(function(i){if(!(i % 2)) $(this).addClass('even')});
                    });

                    self.element.resize();

                    self.animating && setTimeout(function(){
                        self.element.css({minWidth: self.element.width()});
                    }, 50);
                },
                canceled: function(why){
                    --self.n_pending_requests || self.element.trigger('loading', false);
                },
                error: function(json){
                    self._reset(Log.formatError(json));
                    --self.n_pending_requests || self.element.trigger('loading', false);
                }
            });

        },

        destroy: function(){
            this.element.empty().removeClass('probe');
        }

    };

    $.widget('ui.probe', Probe);
});
