import h from 'virtual-dom/h'
import Button from 'grid/button'
import mixins from 'mixins'

var handlers = {}

function Form(args) {

	var o = this.options = extend({
		el: null, // query to find the form element
		ok: null, // callback function
		show_ok: true,
		ok_label: 'Ok'
	}, args)

	this.items = o.items.map(function(v){ return new FormItem(v, o.values[v.name], o.el) });
}

Form.prototype = extend({
	render: function() {
		var buttons = []
		if (this.options.show_ok) {
			buttons.push(new Button({
				text: this.options.ok_label,
				show_text: true,
				on_click: this.bind(this.submit)
			}).render())
		}
		var button_row = h('.buttons', buttons)

		var items = this.items.map(function(item){ console.log(' * item=', item); return handlers[item._item.type].render(item) })
		items.push(button_row)

		return h('.form', items)
	},

	collect: function() {
		var result = {};
		var ok = true;

		this.items.forEach(function(item) {
			var val = item.collect();
			result[item.name()] = val;
			item.error('');
			if (!item.check(val)) { ok = false; }
		})

		return {result: result, ok: ok}
	},

	submit: function(){
		var c = this.collect();
		if (c.ok) {
			this.options.ok(c.result);
			return c.result;
		}
	}
}, {bind: mixins.bind})

Form.add_handler = function(name, handler, _prototype) {
	handler.prototype = _prototype ? (new self[_prototype + 'Handler']()) : new Handler();
	handlers[name] = new handler();
}

function FormItem(item, value, el) {
	this._item = item;
	this._value = value;
	this._el = el; // hmmmm...

	this._handler = item.handler || handlers[item.type];
	if(!this._handler) throw new Error('no form handler for type ' + item.type)
}

FormItem.prototype = {
	values: function() {
		return this._item.values
	},

	name: function() {
		return this._item.name
	},

	value: function() {
		var item = this._item
		return ((item.type === 'float' && typeof this._value !== 'undefined') || this._value) ? this._value : item.value;
	},

	defaults: function () {
		var s = this.value;
		if(s === undefined || s === null) { s = this._item['default'];  }

		while (typeof(s) === 'function') { s = s.apply(this); }

		if(s === undefined || s === null) { s = [];  }

		if (typeof(s) === 'object') { return s; }

		return [s];
	},

	label: function (name, i) {
		var s = this._item.labels, n = name;

		while (typeof(s) === 'function') s = s(name);
		if (typeof(s) === 'string') return s;

		if (Array.isArray(s)) return this._item.labels[i];

		if (typeof(s) === 'object') return s[name];

		if (name === undefined || name === null) {
			name = n;
			if (name === undefined || name === null) {
				name = "?";
			}
		}

		return name;
	},

	collect: function() {
		return this._handler.collect(this, this._el);
	},

	check: function(value) {
		return this._handler.check(this, value);
	},

	multiple: function() {
		return this._item.multiple
	},

	error: function(msg) {
		//this.error_text.html(msg); // TODO
	}
}

function Handler() {
	this.help = function (item) {
		return item.get_help();
	}

	this.check = function () {
		return true;
	}

	return true;
}

export default Form;
